
function pick (x, y)
   -- set the previous picked obj back to regular color
   if picked then
      local picked_obj = objects[picked]
      sb:setColor(255, 255, 255)
      sb:setq(picked_obj.sb_id, quads[picked_obj.id], picked_obj.x, picked_obj.y)
   end

   picked = nil
   for i = #objects, 1, -1 do
      local obj = objects[i]
      local quad = quads[obj.id]
      local qx, qy, w, h = quad:getViewport()
      if x >= obj.x and y >= obj.y and x < obj.x + w and y < obj.y + h then
         local _, _, _, a = data:getPixel(x - obj.x + qx, y - obj.y + qy)
         if a > 0 then
            picked = i
            break
         end
      end
   end

   -- dye currently picked object red
   if picked then
      local picked_obj = objects[picked]
      sb:setColor(255, 0, 0)
      sb:setq(picked_obj.sb_id, quads[picked_obj.id], picked_obj.x, picked_obj.y)
   end
end

function love.update ()
   pick(love.mouse.getPosition())
end

function load_quads (t)
   quads = {}
   for k, v in pairs(t) do
      local args = {unpack(v)}
      args[5] = 512
      args[6] = 512
      quads[#quads+1] = love.graphics.newQuad(unpack(args))
   end
end

function love.load ()
   local emoticon_fns = love.filesystem.enumerate("emoticons/")
   require "quads" -- this loads a bunch of quads into global variable 'quads'
   data = love.image.newImageData "sheet.png"
   img  = love.graphics.newImage(data)
   objects, sb = generate_objects(10000)
   love.graphics.setCaption("Picking test with "..#objects.." objects")
end

local random = math.random
function generate_objects (n)
   local sb = love.graphics.newSpriteBatch(img, n)
   sb:bind()
   local w = love.graphics.getWidth()
   local h = love.graphics.getHeight()
   local objects = {}
   for i = 1, n do
      local id = random(#quads)
      local x = random(w)
      local y = random(h)
      local sb_id = sb:addq(quads[id], x, y)
      objects[i] = {
         id = id,
         x  = x,
         y  = y,
         sb_id = sb_id,
      }
   end
   sb:unbind()
   return objects, sb
end

function love.draw ()
   love.graphics.draw(sb)
end

